// simple.frag

#version 120
#extension GL_EXT_gpu_shader3 : enable

// fragment shader of phong shader

uniform int lightNumber;
uniform int lightRecieve[7];

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int useTexAlpha;
uniform int dissolveMode;
uniform int bFlat;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

varying vec3 position;
varying vec3 normal;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

//##FUNC_SPOT_EFFECT##

//##FUNC_NORMAL##

void main (void)
{
    vec4 texColor = vec4(1.0,1.0,1.0,1.0); 
    if (textureMode == 1)
	texColor = texture2D(textureImage, gl_TexCoord[0].st);
        
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (useTexAlpha == 1){
	alpha *= texColor[3];
	if (alpha == 0) discard;
    }

    //vec3 fnormal = normalize(normal);
    vec3 fnormal = normalize(GetNormal());

    //##APPLY_SMOOTHCLIPPING##

    //##APPLY_ENSEMBLE_DITHERING##
    
    //if (alpha > 1.0) alpha = 1.0;
    if (alpha > 1.0){ 
	texColor[0] *= alpha;
	texColor[1] *= alpha;
	texColor[2] *= alpha; 
	alpha = 1.0;
    }
	
    vec4 diffuseTotal = vec4(0, 0, 0, 1);
    vec4 specularTotal = vec4(0, 0, 0, 1);

    vec3 viewVec = -normalize(position);
    vec4 ambientClr = vec4(0, 0, 0, 0);
    
    for (int i = 0; i < lightNumber; i++){
	if (lightRecieve[i] == 0) continue;

	ambientClr += gl_FrontLightProduct[i].ambient;

	vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position[3];
	float dis = length(lightVec);
	lightVec = normalize(lightVec);

	float diffuse = dot(lightVec, fnormal);
	
	if (diffuse > 0.0){
	    float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
				     -lightVec);
	    float lightFactor = GetSpotEffect(i, spotAngleCos);
	    
	    lightFactor /= (gl_LightSource[i].constantAttenuation +
			    gl_LightSource[i].linearAttenuation * dis +
			    gl_LightSource[i].quadraticAttenuation * dis * dis);

	    vec3 reflectVec = reflect(-lightVec, fnormal);
	    float specular = 
		pow(max(dot(viewVec, reflectVec), 0.0),
		    gl_FrontMaterial.shininess) * lightFactor;
	    diffuseTotal += diffuse * vec4(gl_LightSource[i].diffuse.rgb, 0) * lightFactor;
	    specularTotal += specular * vec4(gl_LightSource[i].diffuse.rgb, 0);
	}
    }

    gl_FragColor = ambientClr + 
	gl_FrontMaterial.diffuse * diffuseTotal +
	gl_FrontMaterial.specular * specularTotal;
    gl_FragColor *= texColor;

    //##APPLY_FOG_EFFECT##

    gl_FragColor[3] = alpha;
}
