#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

uniform int lightNumber;
uniform int lightRecieve[7];

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int fogMode;
uniform int dissolveMode;

uniform vec2 ensembleParamA;
uniform float ensembleParamB;
uniform vec2 ensembleParamC;

uniform float projInv11;
uniform float projInv15;

in vec3 position;
in vec4 color;
in vec4 colorDiffuse;
in vec4 colorSpecular;
in vec2 center;
in float radius;
flat in int blend;
flat in int pseudoSphere;
flat in int boxPoint;
flat in int textureImageIndex;
//flat in int paramBitInt;

//##FUNC_SMOOTHCLIPPING##

//##FUNC_ENSEMBLE_DITHERING##

//##FUNC_FOG_EFFECT##

void main (void)
{
    vec4 texColor;
    vec2 coordXY = (gl_FragCoord.xy - center)/radius;

    vec4 clrBase = color;
    if (pseudoSphere == 1)
	clrBase = vec4(1, 1, 1, 1);

    switch(textureImageIndex){
    case 0:
	if (textureSize(textureImage[0], 0).x > 1)
	    texColor = texture2D(textureImage[0], coordXY * 0.5 + vec2(0.5, 0.5)) * clrBase;
	else{
	    if (boxPoint == 0 &&
		distance( gl_FragCoord.xy, center ) >= radius ) discard;
	    texColor = clrBase;
	}
	break;
    case 1:
	if (textureSize(textureImage[1], 0).x > 1)
	    texColor = texture2D(textureImage[1], coordXY * 0.5 + vec2(0.5, 0.5)) * clrBase;
	else{
	    if (boxPoint == 0 &&
		distance( gl_FragCoord.xy, center ) >= radius ) discard;
	    texColor = clrBase;
	}
	break;
    case 2:
	if (textureSize(textureImage[2], 0).x > 1)
	    texColor = texture2D(textureImage[2], coordXY * 0.5 + vec2(0.5, 0.5)) * clrBase;
	else{
	    if (boxPoint == 0 &&
		distance( gl_FragCoord.xy, center ) >= radius ) discard;
	    texColor = clrBase;
	}
	break;
    default:
	if ( boxPoint == 0 &&
	     distance( gl_FragCoord.xy, center ) >= radius ) discard;
	texColor = clrBase;
	break;
    }

    float alpha = texColor[3];
    if (alpha == 0) discard;

    //##APPLY_SMOOTHCLIPPING##

    //##APPLY_ENSEMBLE_DITHERING##

    if (pseudoSphere == 1){

	vec4 ambientClr = vec4(0, 0, 0, 0);
	
	vec3 diffuseTotal = vec3(0.0, 0.0, 0.0);
	vec3 specularTotal = vec3(0.0, 0.0, 0.0);
	
	float shininess = 20;
	vec3 normal = vec3(coordXY, 1 - length(coordXY));
	
	vec3 fnormal = normalize(normal);
	vec3 viewVec = -normalize(position);
	    
	for (int i = 0; i < lightNumber; i++){
	    if (lightRecieve[i] == 0) continue;

	    ambientClr += gl_LightSource[i].ambient * color;
	    //ambientClr = gl_LightSource[i].ambient;

	    vec3 lightVec = gl_LightSource[i].position.xyz - position * gl_LightSource[i].position.w;
	    float dis = length(lightVec);
	    lightVec = normalize(lightVec);
	    
	    float diffuse = dot(lightVec, fnormal);
	    
	    if (diffuse > 0.0){
		/*
		  float spotAngleCos = dot(normalize(gl_LightSource[i].spotDirection),
		  -lightVec);
		  float lightFactor = GetSpotEffect(i, spotAngleCos);
		*/
		float lightFactor = 1.0f;
		
		lightFactor /= (gl_LightSource[i].constantAttenuation +
				gl_LightSource[i].linearAttenuation * dis +
				gl_LightSource[i].quadraticAttenuation * dis * dis);
		
		vec3 reflectVec = reflect(-lightVec, fnormal);
		float specular = 
		    pow(max(dot(viewVec, reflectVec), 0.0), shininess) * lightFactor;
		diffuseTotal += diffuse * vec3(gl_LightSource[i].diffuse) * lightFactor;
		specularTotal += specular * vec3(gl_LightSource[i].specular);
	    }
	}

	texColor.rgb *= (ambientClr.rgb + diffuseTotal*colorDiffuse.rgb + specularTotal*colorSpecular.rgb);

    }

    gl_FragColor = texColor;//texColor;//colorCurveColor;//color;

    //##APPLY_FOG_EFFECT##

    if (alpha > 1.0){
	gl_FragColor.rgb *= alpha;
	alpha = 1.0;
    }
    
    if (blend == 0){
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = alpha;
    }
    else{
	gl_FragColor.rgb *= alpha;
	gl_FragColor.a = 0;
    }
}
