#version 130
#extension GL_EXT_gpu_shader3 : enable
#extension GL_EXT_gpu_shader4 : enable

//uniform float pointSize;

uniform sampler2D textureColorCurve;
uniform sampler2D textureImage[3];

uniform int dissolveMode;

uniform vec4 viewport;
uniform float baseDistance;

attribute float pointSize;
attribute float colorCurveS;
attribute float colorCurveT;
attribute float filterAlpha;
attribute int paramBit;

out vec3 position;
out vec4 color;
out vec4 colorDiffuse;
out vec4 colorSpecular;
out vec2 center;
out float radius;
flat out int blend;
flat out int pseudoSphere;
flat out int boxPoint;
flat out int textureImageIndex; // valid 0-3 not use if 4;
//flat out int paramBitInt;

void main(void)
{
    position = vec3(gl_ModelViewMatrix * gl_Vertex);

    vec4 baseColor = texture2D(textureColorCurve, vec2(colorCurveS, colorCurveT));
    baseColor[3] *= filterAlpha;

    vec4 colorDiffuseBase = texture2D(textureColorCurve, vec2(colorCurveS, colorCurveT+0.3333));
    vec4 colorSpecularBase = texture2D(textureColorCurve, vec2(colorCurveS, colorCurveT+0.6666));
    
    
    float alpha0 = baseColor[3];

    if (dissolveMode == 1)
	if (baseColor[3] < 0.015)
	    alpha0 = 1;
    
    if (dissolveMode == 2)
	alpha0 = 1;
    
    color = vec4(baseColor[0] * alpha0, baseColor[1] * alpha0, baseColor[2] * alpha0, baseColor[3]);
    colorDiffuse = vec4(colorDiffuseBase[0] * alpha0, colorDiffuseBase[1] * alpha0, colorDiffuseBase[2] * alpha0, baseColor[3]);
    colorSpecular = vec4(colorSpecularBase[0] * alpha0, colorSpecularBase[1] * alpha0, colorSpecularBase[2] * alpha0, baseColor[3]);
    
    
    gl_Position = ftransform();

    /*
    float theta = atan(xydistance/gl_Position.w);
    float factor = 1.0 / cos(theta);
    */

    if (baseDistance != 0){
	gl_PointSize = pointSize / gl_Position.w * baseDistance /** factor */;
	radius = pointSize / gl_Position.w * 0.5 * baseDistance /** factor */;
    }
    else{
	gl_PointSize = pointSize /** factor*/;
	radius = pointSize * 0.5 /** factor*/;
    }
    
    blend = paramBit % 2;
    pseudoSphere = (paramBit / 2) % 2;
    boxPoint = (paramBit / 64) % 2;

    if ((paramBit / 8) % 2 == 0)
	textureImageIndex = 4;
    else
	textureImageIndex = (paramBit / 16) % 4;
    
    center = vec2((gl_Position.x / gl_Position.w * viewport[2] + viewport[2])/2 + viewport[0],
		  (gl_Position.y / gl_Position.w * viewport[3] + viewport[3])/2 + viewport[1]);
}
